#include "doomtype.h"
#include "doomstat.h"
#include "d_player.h"
#include "g_game.h"
#include "d_net.h"
#include "p_local.h"
#include "s_sound.h"
#include "gi.h"
#include "i_net.h"
#include "i_system.h"
#include "c_dispatch.h"
#include "st_stuff.h"
#include "m_argv.h"
#include "cl_main.h"
#include "p_effect.h"
#include "c_console.h"


buf_t     net_buffer;

bool      noservermsgs;
int       last_received;

BOOL      connected;
netadr_t  serveraddr; // address of a server


// some doom functions (not csDoom)
void G_PlayerReborn (int player);
void CL_SpawnPlayer();
void P_KillMobj (AActor *source, AActor *target, AActor *inflictor);
void P_SetPsprite (player_t *player, int position, statenum_t stnum);
void P_ExplodeMissile (AActor* mo);
void G_SetDefaultTurbo (void);
void P_FireWeapon (player_t *player);


void Host_EndGame(char *msg)
{
    Printf(PRINT_HIGH, msg);
    memset (&serveraddr, 0, sizeof(serveraddr));
    connected = false;

	for (int i=0; i<MAXPLAYERS; i++)
		playeringame[i] = 0;

	gameaction = ga_fullconsole;
}


void CL_QuitNetGame(void)
{
    MSG_WriteByte(&net_buffer, clc_disconnect);
    
    NET_SendPacket(net_buffer.cursize, net_buffer.data, serveraddr);
    SZ_Clear(&net_buffer);

    memset (&serveraddr, 0, sizeof(serveraddr));
    connected = false;
	gameaction = ga_fullconsole;
}

//
// CL_DisconnectClient
//
void CL_DisconnectClient(void)
{
	int who = MSG_ReadByte();	
	playeringame[who] = false;

    P_DisconnectEffect (players[who].mo);
}

/////// CONSOLE COMMANDS ///////

BEGIN_COMMAND (connect)
{
	if (connected)
		CL_QuitNetGame();

	if (argc > 1)
	{
	   NET_StringToAdr (argv[1], &serveraddr);

       if (!serveraddr.port)
           I_SetPort(serveraddr, SERVERPORT);
	}
}
END_COMMAND (connect)


BEGIN_COMMAND (disconnect)
{
   CL_QuitNetGame();
}
END_COMMAND (disconnect)


BEGIN_COMMAND (reconnect)
{
    MSG_WriteByte(&net_buffer, clc_disconnect);    
    NET_SendPacket(net_buffer.cursize, net_buffer.data, serveraddr);
    SZ_Clear(&net_buffer);
    connected = false;
	gameaction = ga_fullconsole;
}
END_COMMAND (reconnect)


BEGIN_CUSTOM_CVAR (cl_predict_players, "1", CVAR_ARCHIVE)
{
	if (var.value > 1)
	{
		var.Set (1);
	}
}
END_CUSTOM_CVAR (cl_predict_players)


BEGIN_CUSTOM_CVAR (rate, "2500", CVAR_ARCHIVE)
{
	if (var.value < 100)
	{
		var.Set (100);
	}
	else if (var.value > 100000)
	{
		var.Set (100000);
	}
	else if (connected)
	{
		MSG_WriteByte(&net_buffer, clc_rate);
		MSG_WriteLong(&net_buffer, (int)var.value);
	}
}
END_CUSTOM_CVAR (rate)

BEGIN_CUSTOM_CVAR (cl_maxcorpses, "15", CVAR_ARCHIVE)
{
	if (var.value < 5)
		var.Set (5);

	if (var.value > 100)
		var.Set (100);
}
END_CUSTOM_CVAR (cl_maxcorpses)

/////////////////////////////////


void CL_MoveThing(AActor *mobj, fixed_t x, fixed_t y, fixed_t z)
{
   P_CheckPosition (mobj, x, y);
   mobj->UnlinkFromWorld ();

   mobj->x = x;
   mobj->y = y;
   mobj->z = z;
   mobj->floorz = tmfloorz;
   mobj->ceilingz = tmceilingz;

   mobj->LinkToWorld ();
}


void CL_SendUserInfo(void)
{
   player_t *p;

   p = &players[consoleplayer];

   MSG_WriteByte(&net_buffer, clc_userinfo);
   MSG_WriteString(&net_buffer, p->userinfo.netname);
   MSG_WriteString(&net_buffer, p->userinfo.team);
   MSG_WriteLong(&net_buffer, p->userinfo.gender);
   MSG_WriteLong(&net_buffer, p->userinfo.color);
   MSG_WriteLong(&net_buffer, p->userinfo.aimdist);
}


void CL_SetupUserInfo(void)
{
    player_t *p;
    byte      who;

    who = MSG_ReadByte();

    p = &players[who];

    strcpy(p->userinfo.netname, MSG_ReadString() );
    strcpy(p->userinfo.team, MSG_ReadString() );
	p->userinfo.gender = MSG_ReadLong();
    p->userinfo.color = MSG_ReadLong();
    R_BuildPlayerTranslation (who, p->userinfo.color);
}


//
// CL_UpdateFrags
//
void CL_UpdateFrags(void)
{
	int playernum = MSG_ReadByte();
	players[playernum].fragcount = MSG_ReadShort();
}


AActor* CL_FindThingById(int id)
{
    AActor *mo;
	
    TThinkerIterator<AActor> iterator;
    while ( (mo = iterator.Next() ) )
    {
		if (mo->netid == id)
    	    return mo;
    }

    return 0;
}

void CL_ClearID(int id)
{
    AActor *mo;
	
    TThinkerIterator<AActor> iterator;
    while ( (mo = iterator.Next() ) )
    {
		if (mo->netid == id)
    	    mo->netid = 0;
    }

}

//
// CL_MoveMobj
// 
void CL_MoveMobj(void)
{
	AActor  *mo;
	int      netid;
	fixed_t  x, y, z;

	netid = MSG_ReadShort();
	mo = CL_FindThingById (netid);

	x = MSG_ReadLong();
	y = MSG_ReadLong();
	z = MSG_ReadLong();
	
	if (!mo)
		return;

	CL_MoveThing (mo, x, y, z);
}


//
//  Connecting to a server...
//
bool CL_Connect(void)
{   
   for (int i=0; i<MAXPLAYERS; i++)
       playeringame[i] = false;
   
   CL_ParseCommands();
   if (gameaction == ga_fullconsole) // Host_EndGame was called
	   return false;

   D_SetupUserInfo();
   
   connected = true;
   
   noservermsgs = false;
   last_received = gametic;

   return true;
}


//
// CL_InitNetwork
//
void CL_InitNetwork (void)
{
    char      *v;
    char*     ipaddress;

    netgame = false;  // for old network code
    multiplayer = true;

    v = Args.CheckValue ("-port");
    if (v)
    {
       localport = atoi (v);
       Printf (PRINT_HIGH, "using alternate port %i\n", localport);
    }
    else
       localport = CLIENTPORT;

    // set up a socket and net_message buffer
    InitNetCommon();

    net_buffer.data = new byte[MAX_UDP_PACKET];
    net_buffer.maxsize = MAX_UDP_PACKET;
    SZ_Clear(&net_buffer);

    ipaddress = Args.CheckValue ("-connect");
    if (ipaddress)
    {
       NET_StringToAdr (ipaddress, &serveraddr);

       if (!serveraddr.port)
		   I_SetPort(serveraddr, SERVERPORT);
    }

	G_SetDefaultTurbo ();

    connected = false;
}


void CL_TryToConnect(void)
{
   static int timeout = 0;

   if (!serveraddr.ip[0])
	   return;

   if (!timeout)
   {
	  timeout = 140; // 140 tics = 4 seconds

      Printf(PRINT_HIGH, "connecting to %s\n", NET_AdrToString(serveraddr));
	  MSG_WriteLong(&net_buffer, CHALLENGE); // send challenge
	  MSG_WriteShort(&net_buffer, VERSION); // send client version

	  CL_SendUserInfo(); // send userinfo

  	  MSG_WriteLong(&net_buffer, (int)rate.value);

      NET_SendPacket(net_buffer.cursize, net_buffer.data, serveraddr);
      SZ_Clear(&net_buffer);
   }

   timeout--;
 
   if (NET_GetPacket() )
   {
      if (!CL_Connect())
          memset (&serveraddr, 0, sizeof(serveraddr));
   
      timeout = 0;	    
   }
}


//
// CL_Print
//
void CL_Print (void)
{
	byte level = MSG_ReadByte();
	char *str  = MSG_ReadString();

	Printf (level, str);
    
	if (level == PRINT_CHAT || level == PRINT_TEAMCHAT)
		S_Sound (CHAN_VOICE, gameinfo.chatSound, 1, ATTN_NONE);
}


void CL_UpdatePlayer()
{			
   int       n = MSG_ReadByte();
   int       x, y, z;
   player_t *p = &players[n];
    
   if (!p->mo)
   {
	   for (int i=0; i++; i<29)
		   MSG_ReadByte();
	   return;
   }

   x = MSG_ReadLong();
   y = MSG_ReadLong();
   z = MSG_ReadLong();
   CL_MoveThing(p->mo, x, y, z);
   p->mo->angle = MSG_ReadLong();
   p->mo->frame = MSG_ReadByte();
   p->mo->momx = MSG_ReadLong();
   p->mo->momy = MSG_ReadLong();
   p->mo->momz = MSG_ReadLong();

   //     .   :)
   if (p->mo->frame == PLAYER_FULLBRIGHTFRAME)
	   p->mo->frame = 32773;
   
   p->last_received = gametic;
}


ticcmd_t localcmds[MAXSAVETICS];

void CL_SaveCmd(void)
{
	ticcmd_t *cmd;
    cmd = &localcmds[gametic%MAXSAVETICS];	
	memcpy (cmd, &players[consoleplayer].cmd, sizeof(ticcmd_t));
}

//
// CL_UpdateLocalPlayer
//
void CL_UpdateLocalPlayer(void)
{
	player_t *p;

	p = &players[consoleplayer];

	p->tic = MSG_ReadLong();
	p->real_origin[0] = MSG_ReadLong();
	p->real_origin[1] = MSG_ReadLong();
	p->real_origin[2] = MSG_ReadLong();

	p->real_velocity[0] = MSG_ReadLong();
	p->real_velocity[1] = MSG_ReadLong();
	p->real_velocity[2] = MSG_ReadLong();
}


void CL_ResendSvGametic(void)
{
	int svgametic = MSG_ReadLong();

	MSG_WriteByte (&net_buffer, clc_svgametic);
	MSG_WriteLong (&net_buffer, svgametic);
}


//
// CL_UpdatePing
// Update ping value
//
void CL_UpdatePing(void)
{
	byte      n;

	n = MSG_ReadByte();
	players[n].ping = MSG_ReadByte();
}


//
// CL_SpawnMobj
//
void CL_SpawnMobj(void)
{
	fixed_t  x, y, z;
	short    type;
	short    netid;
	AActor  *mo;

	x = MSG_ReadLong();
	y = MSG_ReadLong();
	z = MSG_ReadLong();
	type = MSG_ReadShort();
	netid = MSG_ReadShort();

	CL_ClearID(netid);

	mo = new AActor (x, y, z, (mobjtype_t)type);
	mo->netid = netid;

	//     , ..    
	//   ,      :)
	if ((mo->flags & MF_MISSILE ) && mo->info->seesound)
		S_Sound (mo, CHAN_VOICE, mo->info->seesound, 1, ATTN_NORM);

	if (mo->type == MT_IFOG)
		S_Sound (mo, CHAN_VOICE, "misc/spawn", 1, ATTN_IDLE);
}

//
// CL_TouchSpecialThing
//
void CL_TouchSpecialThing (void)
{
	int     playernum;
	int     netid;
	AActor* special;

	playernum = MSG_ReadByte();
	netid = MSG_ReadShort();

	special = CL_FindThingById(netid);
	
	if (!special || !players[playernum].mo)
		return;

	P_TouchSpecialThing (special, players[playernum].mo);
}


extern fixed_t cl_viewheight[MAXSAVETICS];
extern fixed_t cl_deltaviewheight[MAXSAVETICS];

//
// CL_SpawnPlayer
//
void CL_SpawnPlayer()
{
	int		  playernum;
	player_t *p;
	AActor	 *mobj, *tfog;
	fixed_t   x, y, z;
	int       id;
	angle_t   angle;
	int       i;


	playernum = MSG_ReadByte();
	id = MSG_ReadShort();
	angle = MSG_ReadLong();
	x = MSG_ReadLong();
	y = MSG_ReadLong();
	z = MSG_ReadLong();

	if (playernum >= MAXPLAYERS)
		return;

	CL_ClearID(id);

	// spawn a teleport fog
	if (playernum != consoleplayer && connected)
	{
		tfog = new AActor (x, y, z, MT_TFOG);
	    S_Sound (tfog, CHAN_VOICE, "misc/teleport", 1, ATTN_NORM);
	}

	playeringame[playernum] = true;
	p = &players[playernum];

	// first disassociate the corpse
	if (p->mo)
	{
		p->mo->player = NULL;
		p->mo->health = 0;
	}

	G_PlayerReborn (playernum);

	mobj = new AActor (x, y, z, MT_PLAYER);

	// set color translations for player sprites
	mobj->translation = translationtables + 256*playernum;
	mobj->angle = angle;
	mobj->pitch = mobj->roll = 0;
	mobj->player = p;
	mobj->health = p->health;
	mobj->netid = id;

	// [RH] Set player sprite based on skin
	mobj->sprite = skins[p->userinfo.skin].sprite;	

	p->mo = p->camera = mobj;	
	p->fov = 90.0f;
	p->playerstate = PST_LIVE;
	p->refire = 0;
	p->damagecount = 0;
	p->bonuscount = 0;
	p->extralight = 0;
	p->fixedcolormap = 0;
	
	p->viewheight = VIEWHEIGHT;
	for (i=0; i<MAXSAVETICS; i++)
	{
		cl_viewheight[i] = VIEWHEIGHT;
		cl_deltaviewheight[i] = 0;
	}

	p->inconsistant = 0;
	p->attacker = NULL;

	p->momx = p->momy = 0;		// killough 10/98: initialize bobbing to 0.

	players[consoleplayer].camera = players[displayplayer].mo;

	// setup gun psprite
	P_SetupPsprites (p);

	// give all cards in death match mode
	for (i = 0; i < NUMCARDS; i++)
		p->cards[i] = true;

	ST_Start ();
}

//
// CL_SetMobjSpeedAndAngle
//
void CL_SetMobjSpeedAndAngle(void)
{
	AActor *mo;
	int     netid;

	netid = MSG_ReadShort();
	mo = CL_FindThingById(netid);

	if (!mo) 
	{
		for (int i=0; i<4; i++)
			MSG_ReadLong();
		return;
	}
	
	mo->angle = MSG_ReadLong();
	mo->momx = MSG_ReadLong();
	mo->momy = MSG_ReadLong();
	mo->momz = MSG_ReadLong();
}

//
// CL_ExplodeMissile
//
void CL_ExplodeMissile(void)
{
    AActor *mo;
	int     netid;

	netid = MSG_ReadShort();
	mo = CL_FindThingById(netid);

	if (!mo) 
		return;

	P_ExplodeMissile(mo);
}


//
// CL_UpdateMobjFrame
//
void CL_UpdateMobjFrame(void)
{
	AActor *mo;
	int     netid;
	byte    frame;

	netid = MSG_ReadShort();
	frame = MSG_ReadByte();

	mo = CL_FindThingById(netid);

	if (!mo) 
		return;

	mo->frame = frame;
}


//
// CL_RemoveMobj
//
void CL_RemoveMobj(void)
{
	AActor *mo;
	int     netid;

	netid = MSG_ReadShort();
	mo = CL_FindThingById(netid);

	if (!mo) 
		return;

	mo->Destroy();
}


//
// CL_DamagePlayer
//
void CL_DamagePlayer(void)
{
	player_t *p;	
	int       temp;
	int       health;
	int       damage;
	byte      playernum;

	playernum = MSG_ReadByte();

	p = &players[playernum];

	p->armorpoints = MSG_ReadByte();
	health         = MSG_ReadShort();

	damage = p->health - health;
	p->mo->health = p->health = health;

	if (p->health < 0)
		p->health = 0;

    if (damage < 0)  // can't be!
		return;

	p->damagecount += damage;

	if (p->damagecount > 100)
		p->damagecount = 100;

	if (playernum == consoleplayer)
	{
		temp = damage < 100 ? damage : 100;
        I_Tactile (40,10,40+temp*2);
	}
}

extern int MeansOfDeath;

//
// CL_KillMobj
//
void CL_KillMobj(void)
{
	AActor *source;
	AActor *target;
	AActor *inflictor;
	
	source = CL_FindThingById (MSG_ReadShort() );
	target = CL_FindThingById (MSG_ReadShort() );
	inflictor = CL_FindThingById (MSG_ReadShort() );
	
	MeansOfDeath = MSG_ReadLong();

	if (!target)
		return;
	
	P_KillMobj (source, target, inflictor);
}


///////////////////////////////////////////////////////////
///// CL_Fire* called when someone uses a weapon  /////////
///////////////////////////////////////////////////////////

//
// CL_FirePistol
//
void CL_FirePistol(void)
{
	int playernum = MSG_ReadByte();
	player_t *player = &players[playernum];
	
	if (playernum != consoleplayer)
		S_Sound (player->mo, CHAN_WEAPON, "weapons/pistol", 1, ATTN_NORM);		
}

//
// CL_FireShotgun
//
void CL_FireShotgun(void)
{
	int playernum = MSG_ReadByte();
	player_t *player = &players[playernum];
	
	// playernum != consoleplayer
	S_Sound (player->mo, CHAN_WEAPON,  "weapons/shotgf", 1, ATTN_NORM);
}

//
// CL_FireSSG
//
void CL_FireSSG(void)
{
	int playernum = MSG_ReadByte();
	player_t *player = &players[playernum];
	
	S_Sound (player->mo, CHAN_WEAPON, "weapons/sshotf", 1, ATTN_NORM);
}


//
// CL_FireChainGun
//
void CL_FireChainGun(void)
{
	int playernum;

	playernum = MSG_ReadByte();
	player_t *player = &players[playernum];

	S_Sound (player->mo, CHAN_WEAPON, "weapons/chngun", 1, ATTN_NORM);
}

/////////////////////////////////////////////////////////

void CL_ChangeWeapon (void)
{
	player_t* player = &players[consoleplayer];
	
	player->pendingweapon = (weapontype_t)MSG_ReadByte();

	// Now set appropriate weapon overlay.
	P_SetPsprite (player,
				  ps_weapon,
				  weaponinfo[player->readyweapon].downstate);
}


//
// CL_StartSound
//
void CL_Sound(void)
{	
	int netid = MSG_ReadShort();
	byte channel = MSG_ReadByte();
	byte sfx_id = MSG_ReadByte();
	byte attenuation = MSG_ReadByte();

	AActor *mo = CL_FindThingById (netid);
	if (!mo)
		return;

	S_SoundID (mo, channel, sfx_id, 1, attenuation);
}


//
// CL_UpdateSector
// Updates floorheight and ceilingheight of a sector.
//
void CL_UpdateSector(void)
{
	unsigned short s = (unsigned short)MSG_ReadShort();
	sector_t *sec = &sectors[s];

	sec->floorheight = MSG_ReadShort() << FRACBITS;
	sec->ceilingheight = MSG_ReadShort() << FRACBITS;	
	P_ChangeSector (sec, -1);
}

void CL_UpdateFloorHeight(void)
{
	unsigned short s = (unsigned short)MSG_ReadShort();
	sector_t *sec = &sectors[s];

	sec->floorheight = MSG_ReadShort() << FRACBITS;
	P_ChangeSector (sec, -1);
}

void CL_UpdateCeilingHeight(void)
{
	unsigned short s = (unsigned short)MSG_ReadShort();
	sector_t *sec = &sectors[s];

	sec->ceilingheight = MSG_ReadShort() << FRACBITS;
	P_ChangeSector (sec, -1);
}

//
// CL_RemoveCorpses
// Removes some corpses
//
void CL_RemoveCorpses (void)
{
	AActor *mo;
	int     corpses = 0;

	if (gametic%35)
	{
		return;
	}
	else
	{
		TThinkerIterator<AActor> iterator;
		while ( (mo = iterator.Next() ) )
		{
			if (mo->type == MT_PLAYER && (!mo->player || mo->health <=0) )
				corpses++;
		}
	}

	if (corpses > cl_maxcorpses.value)
	{
		TThinkerIterator<AActor> iterator;
		while (corpses > cl_maxcorpses.value && (mo = iterator.Next() ) )
		{
			if (mo->type == MT_PLAYER && !mo->player)
			{
				mo->Destroy();
				corpses--;
			}
		}
	}
}

//
// CL_ParseCommands
//
void CL_ParseCommands(void)
{
	 byte   cmd;
	 char  *mapname;

	 
	 while(1)	 
	 {  
		cmd = MSG_ReadByte();

		if (msg_badread)
		{
			CL_QuitNetGame();
			Printf(PRINT_HIGH, "CL_ParseCommands: Bad server message\n");
		}

		if (cmd == svc_endmessage)
		    break;
	    
		switch(cmd)
		{
	    case svc_loadmap:
			mapname = MSG_ReadString();
            G_InitNew (mapname);
			break;
			
		case svc_gameskill:
			gameskill.Set((float)MSG_ReadByte());
			break;

		case svc_dmflags:
			dmflags = MSG_ReadLong();
			break;
			  
		case svc_consoleplayer:
		    displayplayer = consoleplayer = MSG_ReadByte();
		    break;

		case svc_updatefrags:
			CL_UpdateFrags();
			break;

		case svc_moveplayer:			
			CL_UpdatePlayer();
			break;

		case svc_updatelocalplayer:
			CL_UpdateLocalPlayer();
			break;

		case svc_userinfo:  // name, color, team
			CL_SetupUserInfo();
			break;

		case svc_svgametic:
			CL_ResendSvGametic();
			break;

		case svc_updateping:
			CL_UpdatePing();
			break;

		case svc_spawnmobj:
			CL_SpawnMobj();
			break;

		case svc_mobjspeedangle:
			CL_SetMobjSpeedAndAngle();
			break;

		case svc_mobjframe:
			CL_UpdateMobjFrame();
			break;

		case svc_explodemissile:
			CL_ExplodeMissile();
			break;

		case svc_removemobj:
			CL_RemoveMobj();
			break;
		
		case svc_killmobj:
			CL_KillMobj();
			break;

		case svc_movemobj:
			CL_MoveMobj();
			break;

		case svc_spawnplayer:
			CL_SpawnPlayer();
			break;
			
		case svc_damageplayer:
			CL_DamagePlayer();
			break;

		case svc_firepistol:
			CL_FirePistol();
			break;

		case svc_fireshotgun:
			CL_FireShotgun();
			break;

		case svc_firessg:
			CL_FireSSG();
			break;

		case svc_firechaingun:
			CL_FireChainGun();
			break;

//		case svc_fireweapon:
//			P_FireWeapon(&players[consoleplayer]);
//			break;

		case svc_changeweapon:
			CL_ChangeWeapon();
			break;

		case svc_disconnectclient:
			CL_DisconnectClient();
			break;

		case svc_sector:
			CL_UpdateSector();
			break;
		
		case svc_floorheight:
			CL_UpdateFloorHeight();
			break;
		
		case svc_ceilingheight:
			CL_UpdateCeilingHeight();
			break;
			
		case svc_print:
			CL_Print();
			break;

		case svc_startsound:
			CL_Sound();
			break;

		case svc_touchspecial:
			CL_TouchSpecialThing ();
			break;

		case svc_disconnect:
            if (connected)
				Host_EndGame ("Server disconnected\n");
			else
			{
                Host_EndGame ("Server disconnected\n"
                              "Server version may not be compatible");
			}
			return;
		
		case svc_full:           
			Host_EndGame ("Server is full\n");
			return;
		
		case svc_reconnect:
            connected = false;
	        gameaction = ga_fullconsole;
			return;
		
		case svc_exitlevel:
			gamestate = GS_INTERMISSION;
			return;
		
		default:			
			Printf(PRINT_HIGH, "CL_ParseCommands: Unknown server message.\n");
			return;
		}	 
	}
	 
}

extern int outrate;

//
// CL_SendCmd
//
void CL_SendCmd(void)
{		
	ticcmd_t *cmd;
	player_t *p;

	if (gametic < 1 )
		return;

	p = &players[consoleplayer];

	MSG_WriteByte(&net_buffer, clc_move);

    MSG_WriteLong(&net_buffer, gametic); // current tic

    // send the previous cmds in the message, so if the last packet 
    // was dropped, it can be recovered
    cmd = &localcmds[(gametic-1) % MAXSAVETICS];
			
	MSG_WriteByte(&net_buffer, cmd->ucmd.buttons);
	MSG_WriteShort(&net_buffer, p->mo->angle >> 16);
	MSG_WriteShort(&net_buffer, p->mo->pitch >> 16);
	MSG_WriteShort(&net_buffer, cmd->ucmd.forwardmove);
	MSG_WriteShort(&net_buffer, cmd->ucmd.sidemove);

    // send the curent cmds in the message
    cmd = &players[consoleplayer].cmd;


	// from p_user.cpp
	// [RH] Look up/down stuff
	if (dmflags & DF_NO_FREELOOK)
	{
		p->mo->pitch = 0;
	}
	else
	{
		int look = cmd->ucmd.pitch << 16;

		// The player's view pitch is clamped between -32 and +56 degrees,
		// which translates to about half a screen height up and (more than)
		// one full screen height down from straight ahead when view panning
		// is used.
		if (look)
		{
			if (look == -32768 << 16)
			{ // center view
				p->mo->pitch = 0;
			}
			else if (look > 0)
			{ // look up
				p->mo->pitch -= look;
				if (p->mo->pitch < -ANGLE_1*32)
					p->mo->pitch = -ANGLE_1*32;
			}
			else
			{ // look down
				p->mo->pitch -= look;
				if (p->mo->pitch > ANGLE_1*56)
					p->mo->pitch = ANGLE_1*56;
			}
		}
	}


	MSG_WriteByte(&net_buffer, cmd->ucmd.buttons);
	MSG_WriteShort(&net_buffer, (p->mo->angle + (cmd->ucmd.yaw << 16)) >> 16);
	MSG_WriteShort(&net_buffer, p->mo->pitch >> 16);
	MSG_WriteShort(&net_buffer, cmd->ucmd.forwardmove);
	MSG_WriteShort(&net_buffer, cmd->ucmd.sidemove);

	MSG_WriteByte(&net_buffer, clc_endmessage);
    NET_SendPacket(net_buffer.cursize, net_buffer.data, serveraddr);
	outrate += net_buffer.cursize;
    SZ_Clear(&net_buffer);
}
